import org.gradle.internal.declarativedsl.parsing.main
import org.jetbrains.kotlin.gradle.internal.wrapPluginOptions

plugins {
    alias(libs.plugins.android.application)
    alias(libs.plugins.kotlin.android)
    id("dagger.hilt.android.plugin")
    id("kotlin-kapt")
    id("com.google.protobuf")
}

android {
    namespace = "com.sd.cavphmi"
    compileSdk = 35

    defaultConfig {
        applicationId = "com.sd.cavphmi"
        minSdk = 29
        targetSdk = 35
        versionCode = 1
        versionName = "1.0"

        testInstrumentationRunner = "androidx.test.runner.AndroidJUnitRunner"
    }

    buildTypes {
        release {
            isMinifyEnabled = false
            proguardFiles(
                getDefaultProguardFile("proguard-android-optimize.txt"),
                "proguard-rules.pro"
            )
        }
    }
    compileOptions {
        sourceCompatibility = JavaVersion.VERSION_11
        targetCompatibility = JavaVersion.VERSION_11
    }
    kotlinOptions {
        jvmTarget = "11"
    }
    buildFeatures {
        dataBinding = true
    }
    lint {
        abortOnError=false
        checkReleaseBuilds=false
    }
    sourceSets {
        //配置 protoc 编译器


    }
}

protobuf {
    //配置 protoc 编译器
//    protoc {
//        artifact = "com.google.protobuf:protoc-gen-javalite:3.0.0"
//    }
    plugins {
        // The codegen for lite comes as a separate artifact
//       artifacts {
//           "com.google.protobuf:protoc-gen-javalite:3.0.0"
//       }
    }
    //配置生成目录，编译后会在 build 的目录下生成对应的java文件
    generateProtoTasks {
        all().forEach { task->
            task.builtins {
//                remove(com.google.protobuf.gradle.ProtobufPlugin.j)
//                remove(java)
            }
            task.builtins {
                java {

                }
            }
        }
    }
}

dependencies {
    implementation(fileTree(mapOf("dir" to "libs","include" to listOf("*.jar", "*.aar"))))
    implementation(libs.androidx.core.ktx)
    implementation(libs.androidx.appcompat)
    implementation(libs.material)
    implementation(libs.androidx.activity)
    implementation(libs.androidx.constraintlayout)
    testImplementation(libs.junit)
    androidTestImplementation(libs.androidx.junit)
    androidTestImplementation(libs.androidx.espresso.core)

    implementation("com.google.protobuf:protobuf-java:3.19.2")
//    implementation("com.google.protobuf:protobuf-lite:3.0.0")
//    implementation("com.google.protobuf:protobuf-lite:3.0.0")
//    implementation("com.google.protobuf:protobuf-kotlin-lite:4.31.1")

    //java web socket
    implementation("org.java-websocket:Java-WebSocket:1.6.0")
    //proto
    // Hilt dependencies
    val hilt_version = "2.56.1"
    implementation("com.google.dagger:hilt-android:$hilt_version")
    kapt("com.google.dagger:hilt-android-compiler:${hilt_version}")
    // ViewModel
    implementation("androidx.lifecycle:lifecycle-viewmodel-ktx:2.6.2")
    //fragment-ktx
    implementation("androidx.fragment:fragment-ktx:1.8.0")
    // 权限请求框架
    implementation("com.guolindev.permissionx:permissionx:1.8.1")
    //加密库
    implementation("androidx.security:security-crypto:1.0.0")
//    implementation("com.gyf.immersionbar:immersionbar:2.3.3-beta15")

    //网络
    implementation("io.reactivex.rxjava3:rxjava:3.0.8")
    implementation("io.reactivex.rxjava3:rxandroid:3.0.0")

    implementation("com.squareup.retrofit2:retrofit:2.9.0")
    implementation("com.squareup.retrofit2:converter-gson:2.9.0")
    implementation("com.squareup.retrofit2:adapter-rxjava3:2.9.0")
    implementation("com.squareup.retrofit2:converter-scalars:2.9.0")
    implementation("com.squareup.okhttp3:logging-interceptor:4.9.0")
}